;;
;; 'look' command interface.
;; Author: S.Namba
;; splitted from my-lib.el on Wed Oct 21 18:14:57 1992
;;
(defun look (wordpsx &optional flag)
  "Interface to 'look' command.
Invokes 'look' command using call-process.
You can input any option flag preceeding target PREFIX[/SUFFIX].
For example, if you want to look up word starting with 'dir',
input prefix 'dir' to its prompt.  You can optionally specify word 
suffix; If you want to look up word starting with 'dir' and ending
with 'ry', input 'dir/ry' to its prompt. 
Given numeric ARG, this command does not clean up its output buffer."
  (interactive "sWordprefix[/suffix]: \nP")
  (let ((buf (get-buffer-create " *looking-dictionary*")))
    (save-excursion
      (set-buffer buf)
      (goto-char (point-min))
      (if (null flag) (erase-buffer)))
    (call-process "look" nil buf nil wordpsx)
    (if (save-excursion
	  (set-buffer buf)
	  (> (buffer-size) 0))
	(set-window-start (display-buffer buf) 1)
      (message "(Look command completed with no output)"))))

(defvar look-case
  "Case of the primitive word." nil)
(defvar last-prim-word
  "The primitive word passed to last 'look' command. Downcased." nil)

(defun look-backward-word ()
  "Same as backward-word function except it ignores [/'] as a word delimiter."
  (backward-word 1)
  (if (not (bobp))
      (progn (backward-char 1)
	     (if (looking-at "[/']")
		 (backward-word 1)
	       (forward-char 1)))))

(defun look-here (&optional flag)
  "Interface to 'look' command.
Invokes 'look' command using call-process.
You can input any option flag preceeding target PREFIX[/SUFFIX].

For example, if you want to insert a word starting with 'dir', input
prefix 'dir' to its prompt.  You can optionally specify word suffix;
If you want to insert a word 'directory', which starting with 'dir'
and ending with 'ry', input 'dir/ry' to its prompt.

Given numeric ARG, it inserts next candidate."
  (interactive "*P")
  (if flag
      ;; inserts next candidate from " *looking-here*" buffer...
      (let ((the-word-start
	     (save-excursion
	       (look-backward-word)
	       (point)))
	    (buf " *looking-here*"))
	(if (get-buffer buf)
	    (if (save-excursion (set-buffer buf) ;;
				(forward-line 1)
				(= (point) (point-max)))
		(progn (message "(No more candidate for %s)" last-prim-word)
		       (save-excursion (set-buffer buf)
				       (delete-window)))
	      (let ((the-next-word (save-excursion
				     (set-buffer buf)
				     (buffer-substring (point)
						       (save-excursion
							 (forward-line 1)
							 (backward-char 1)
							 (point))))))
		(if (save-excursion
		      (goto-char the-word-start)
		      (looking-at the-next-word))
		    (message
		     "(That's the candidate '%s' from the 'look'.)"
		     the-next-word)
		  ;; Insert a candidate, make small window for the candidates.
		  (progn
		    (delete-region (point) the-word-start)
		    (insert the-next-word)
		    ;; case fiddle...
		    (if (not (equal look-case 'down))
			(save-excursion
			  (look-backward-word)
			  (cond ((equal look-case 'cap)
				 (capitalize-word 1))
				((equal look-case 'up)
				 (upcase-word 1)))))
		    ;; display the candidate's buffer
		    (if (not (get-buffer-window buf))
			(progn
			  (split-window-horizontally)
			  (enlarge-window-horizontally (- (window-width) 16))
			  (other-window 1)
			  (switch-to-buffer buf)
			  (other-window -1)))
		    ;; (next-window) should be " *looking-here*"
		    ;; This assumption may cause a bug.
		    ;; There is no such function as current-window ?
		    (set-window-start
		     (next-window) (save-excursion
				     (set-buffer buf) (point)))))))
	(message "(No previous Look.)")))

    ;; Operation without ARG....
    (let* ((buf (get-buffer-create " *looking-here*"))
	   (the-word-start (save-excursion
			     (look-backward-word)
			     (point)))
	   (the-word-prim (buffer-substring the-word-start (point))))
      (setq last-prim-word (downcase the-word-prim))
      (setq look-case
	    (cond ((equal the-word-prim (capitalize the-word-prim)) 'cap)
		  ((equal the-word-prim (upcase the-word-prim)) 'up)
		  (t 'down)))
      (save-excursion
	(set-buffer buf)
	(erase-buffer))
      (call-process "look" nil buf nil last-prim-word)
      ;; Now " *looking-here*" buffer holds all the candidates...
      (if (save-excursion
	    (set-buffer buf)
	    (> (buffer-size) 0))
	  ;; Get first word in the output buffer, and insert it.
	  (let ((the-first-word
		 (save-excursion
		   (set-buffer buf)
		   (goto-char (point-min))
		   (buffer-substring (point)
				     (save-excursion
				       (forward-line 1)
				       (backward-char 1)
				       (point))))))
	    (if (save-excursion
		  (goto-char the-word-start)
		  (looking-at the-first-word))
		(message
		 "(That's the first candidate '%s' from the 'look'.)"
		 the-first-word)
	      ;; if the-first-word was different, insert it.
	      (progn (delete-region (point) the-word-start)
		     (insert the-first-word)
		     (if (not (equal look-case 'down))
			 (save-excursion
			   (look-backward-word)
			   (cond ((equal look-case 'cap)
				  (capitalize-word 1))
				 ((equal look-case 'up)
				  (upcase-word 1))))))))
	(message "(Look command completed with no output)")))))

(global-set-key "\C-xL"		'look)
(global-set-key "\eL"		'look-here)

