;;;
;;; memo.el, version 05, a stuff like `post-it'
;;;
;;;	Seiichi.Namba <pi9s-nnb@asahi-net.or.jp / GBC00441@niftyserve.or.jp>
;;;
;;; 	Started: Mon Jun 19 18:23:45 1995
;;;	Smugged things from occur, occur-mode (replace.el in v19).
;;;
;;;     version 04:
;;;         variable memo-no-newline-yank (default t).
;;;         Insertion of ./*.memo file where * stands for mother file.
;;;         This version was previewed in nifty.funix.linux.
;;;     version 05:
;;;         Clean up. `Modified' flag fiddling.  for C-u 0 M-x memo call.
;;; 
;;; Function `memo' pops up a special buffer "*memo*BUFNAME" for a current
;;; buffer, and inserts current line in the beginning of the special
;;; buffer.  Non-nil argument attached `memo' command, it sends current
;;; region to *memo* buffer.
;;; 
;;; The *memo* buffer is automatically set into "Memo" mode.  In this
;;; mode, a command C-c C-c (memo-yank-to-mother-buffer: default
;;; binding in memo-mode-map) sends the current line in *memo* buffer
;;; back to its `mother buffer'.
;;; 
;;; A file name with ".memo" extension is automatically associated to
;;; the `memo' buffer as "viisited file name". If the file already exists,
;;; it is loaded to `memo' buffer on startup.
;;;
;;; The `mother buffer' means the buffer where you issued `memo'
;;; command to create the *memo* buffer.
;;; 
;;; Any emacs buffer can own one *memo* buffer (all *memo* buffer are
;;; `buffer local') each. There might be some implementation to have a
;;; global *memo* buffer for every buffer for emacs, but current
;;; version has no facility to handle such `global' *memo* buffer
;;; (sorry, in advance).
;;;
;;; You can use frame facility of V19, and a mouse command [mouse-2]
;;; (memo-mouse-yank-to-mother-buffer), which is an equivalent of the
;;; C-c C-c (memo-yank-to-mother-buffer).
;;;  
;;; There isn't any other commands in *memo* mode. You can use *memo*
;;; buffer as an ordinary buffer 'cept the command(s) above.
;;;

;;; 

;;; Sample config in your ~/.emacs file:
;(autoload 'memo "memo" nil t)
;(global-set-key "\M-M" 'memo)		;; you got meta key, don't you ?
;
;;; If your emacs is version 19.xx, put these in .emacs to use special frame. 
;(if (>= (string-to-int emacs-version) 19)
;    (progn (setq memo-frame-parameters 
;		 '((name . "*memo*emacs")
;		   (width . 50) (height . 18) (minibuffer) (menu-bar-lines)))
;	   (setq memo-use-special-frame t)))
;
;;; Additionally, setup special frame config like this:
;(setq special-display-regexps
;      '("\\*Help" "\\*[Ss]hell" "\\*Mini" "\\*RMAIL" "\\*diff"
;       "\\*memo\\*"			;; This is it.
;	"\\*compil" "\\*man"    
;	"\\*Completions" "\\*Occur"))
;;; You can use special-display-buffer-names to specify "*memo*" buffer.
;
;;; If you want to add lines always on top of *memo* buffer,
;       (setq memo-add-on-top t)
;
;;; The memo frame is always named as "*memo*emacs"
;   if the special frame is used, so
;   the frame property can be controlled with window manager.
;   For instance, use these lines ~/.fvwmrc:
;       StaysOnTop *memo*emacs
;       Sticky *memo*emacs 
;   StayOnTop implementation may be slightly complicated. RTFM. or,
;       get used to it.
;;; 

(defconst memo-buffer-basename "*memo*")
(defvar memo-buffer nil)
(defvar memo-mode-map ())
(if memo-mode-map
    ()
  (setq memo-mode-map (make-sparse-keymap))
  (if (>= (string-to-int emacs-version) 19)
      (define-key memo-mode-map [mouse-2] 'memo-mouse-yank-to-mother-buffer))
  (define-key memo-mode-map "\C-c\C-c" 'memo-yank-to-mother-buffer))
;; These two will be used locally.
(defvar memo-mother-buffer nil)
(defvar memo-file nil)

;; Four preferences:
(defvar memo-frame-parameters
      '((name . "*memo*emacs")
	(width . 50) (height . 18) (minibuffer) (menu-bar-lines)))
(defvar memo-use-special-frame nil "Use frame for *memo* buffer.")
(defvar memo-no-newline-yank t)
(defvar memo-add-on-top nil "Add on top of *memo* buffer.")

(defun memo (arg)
  "Poup memo-mode buffer *memo* and inserts current line in it.
With numeric arg 0, just opens memo window.
With non-nil ARG, inserts current region to memo buffer."
  (interactive "P")
  (let* ((mothbuf (current-buffer))
	 (memofile
	  (concat 
	   (or (buffer-file-name)
	       (buffer-name)) ".memo"))
	 (memo-buf (concat memo-buffer-basename (buffer-name)))
	 (already (if (get-buffer memo-buf) t nil))
	 (just-open (= 0 (prefix-numeric-value arg)))
	 (str (if (and (not just-open) arg) 
		   (buffer-substring (region-beginning) (region-end))
		 (memo-get-current-line nil))))
    (pop-to-buffer 
     (get-buffer-create memo-buf))

    ;; Now we are in memo-buf
    ;; memo-mode-setup just for the first time.
    (or already
	(memo-mode-setup mothbuf memofile))
    ;; This may have to be done in memo-mode-setup, I don't know..
    (if (and (not already) (file-exists-p memofile))
	 (insert-file-contents memofile t))
	       
    ;; Dirty Works All Done. User inserting starts here. So:
    (set-buffer-modified-p nil)
    ;; After all it will be `modified' unless just-open, though.

    (memo-move-to-add)
    (or just-open
	(if (string-equal "" str)
	    ()
	  ;(setq bg (point))
	  (insert str)
	  ;(setq end (point))
	  ;To heavy, even post-inserting-back process is not implemented
	  ;(if (>= (string-to-int emacs-version) 19)
	  ;    (put-text-property bg end 'mouse-face 'highlight))
	  (or (string-equal (substring str -1) "\n")
	      (insert "\n"))
	  (memo-move-to-add)))))

(defun memo-move-to-add ()
  (if memo-add-on-top
      (goto-char (point-min))
    (goto-char (point-max))))

;;debugger
;;(message "%s %s" memo-file memo-mother-buffer)

(defun memo-mode-setup (mbuf mfile)
  "Not an interactive `mode' function, used by memo,
because `memo' buffer should always be associated with it's mother buffer."
  (let ((memo-mother-buffer-name (buffer-name mbuf)))
    (kill-all-local-variables)
    (use-local-map memo-mode-map)
    (make-local-variable 'memo-mother-buffer)
    (make-local-variable 'memo-file)
    (setq memo-mother-buffer mbuf)
    (setq memo-file mfile)

    (setq major-mode 'memo-mode)
    (setq mode-name "Memo")
    ;; This changes buffer to `modified' status.
    (set-visited-file-name
     (concat 
      (or (buffer-file-name memo-mother-buffer)
	  memo-mother-buffer-name) ".memo"))
    (rename-buffer
     (concat memo-buffer-basename memo-mother-buffer-name))

    (if memo-use-special-frame
	(modify-frame-parameters (selected-frame) memo-frame-parameters))

    (run-hooks 'memo-mode-hook)))

;; Not yet !
;; Experimental standalone `Memo' mode.  Situation is really tangled:
;; e.g., how should I handle the situation with no mother buf/file ?
;; Ask user to read mother file ?
;; Did I fucked up fundamental design at all?
(defun memo-mode ()
  "Experimental standalone `Memo' mode."
;  (interactive)
  (let* (mbuf
	 mfile
	 (fname (buffer-file-name))
	 (matched-point (string-match "\\.memo" fname)))
    (if (not matched-point)
	(error "`.memo' extension needed for filename.")
      ;; Here are only codes for a buffer with file !
      (if fname
	  (progn
	    ;; I don't know if these exists.
	    (setq mfile
		  (substring fname 0 
			     matched-point))
	    (setq mbuf (get-buffer (file-name-nondirectory mfile))))))
    (message "%s %s" mbuf mfile)))

(defun memo-mouse-yank-to-mother-buffer (event)
  "Inserts current memoline into current position of memo-mother-buffer."
  (interactive "*e")
  (if (get-buffer memo-mother-buffer)
      (let (str
	    (cpos
	     (save-excursion
	       (set-buffer memo-mother-buffer) (point))))
	(save-excursion
	  (set-buffer (window-buffer (posn-window (event-end event))))
	  (goto-char (posn-point (event-end event)))
	  (setq str (memo-get-current-line t)) )

	;; truncate \n, Sun Apr 14 15:38:57 1996
	(if memo-no-newline-yank
	    (setq str (substring str 0 -1)))

	(pop-to-buffer memo-mother-buffer)
	(goto-char cpos)
	(insert str))
    (ding)
    (message "mother-buffer disappeared")))

(defun memo-yank-to-mother-buffer (arg)
  "Inserts current memoline into current position of memo-mother-buffer.
If wit non-nil ARG, insert without newline."
  (interactive "*P")
  (if (get-buffer memo-mother-buffer)
      (let ((str (memo-get-current-line (not arg)))
	    (cpos
	     (save-excursion
	       (set-buffer memo-mother-buffer) (point))))

	;; truncate \n, Sun Apr 14 15:38:57 1996
	(if memo-no-newline-yank
	    (setq str (substring str 0 -1)))

	(pop-to-buffer memo-mother-buffer)
	(goto-char cpos)
	(insert str))
    (ding)
    (message "mother-buffer disappeared")))

(defun memo-get-current-line (arg)
  (interactive "P")
  "Returns current line string. If with non-nil ARG, add newline at end."
  (concat 
   (buffer-substring (save-excursion (beginning-of-line 1) (point))
		     (save-excursion (end-of-line 1) (point)))
   (if arg "\n" nil)))

(provide 'memo)
