;;;
;;; viewhook.el
;;;    makes view-mode compatible to 'less'.
;;;
;;; Mon Jun  1 18:55:55 1992
;;;    1st draft.
;;; Wed Jul 15 18:49:11 1992
;;;    my-view-buffer: Immediately current-buffer goes into view-mode.
;;; Wed Jun 22 17:47:57 1994
;;;    No-exit-on-EOB facility, cleanup.
;;; Mon Jul 31 18:46:40 1995
;;;    Support face-re-search, Clenup for V19 (very rough).
;;; Tue Dec 30 19:49:33 1997
;;;    emacs-20(.2) support.
;;;

;; This may slow down startup, but proprietary view.el has no load-hook.
;; This file overrides View-scroll-lines-forward, so we require now.
(require 'view)

;; require my face-re-search
(if (>= (string-to-int emacs-version) 19)
    (require 'face-re-search))

(if (>= (string-to-int emacs-version) 19)
    (progn
      (global-set-key "\C-x4F"		'view-file-other-window)
      (global-set-key "\C-x4B"		'view-buffer-other-window)))

;; Covering view-buffer.
(global-set-key "\C-xv"		'my-view-buffer)
(defun my-view-buffer ()
  "Sets view-mode to current-buffer."
  (interactive)
  (view-buffer (buffer-name (current-buffer))))

(defvar View-exit-on-EOB nil
  "If nil, View-mode does not exit on EOB.")
;;(fset 'View-quit-on-EOB 'View-exit-on-EOB)

;; Override View-scroll-lines-forward (in view.el) here.
;; View-scroll-lines-backward don't have to be patched because
;; it invokes View-scroll-lines-forward with negative numeric argument.

(defun View-scroll-lines-forward (&optional lines)
  "Scroll forward in View mode. This version does not exit on EOB if View-exit-on-EOB is nil.
No arg means whole window full, or number of lines set by \\[View-scroll-lines-forward-set-scroll-size].
Arg is number of lines to scroll."
  (interactive "P")
;;; Covering "if View-exit-on-EOB" by S.Namba, Tue Jun 21 17:44:02 1994
  (if View-exit-on-EOB
      (if (and (pos-visible-in-window-p (point-max))
	       (not (and (numberp lines) (< lines 0))))
	  (if (>= (string-to-int emacs-version) 19)
	      (view-exit)
	    (exit-recursive-edit))))
;;; Covering "if View-exit-on-EOB" by S.Namba, Tue Jun 21 17:44:02 1994
  (setq lines
	(if lines (prefix-numeric-value lines)
	  (view-scroll-size)))
; (view-last-command 'View-scroll-lines-forward lines)
  (if (>= lines (view-window-size))
      (scroll-up nil)
    (if (>= (- lines) (view-window-size))
	(scroll-down nil)
      (scroll-up lines)))
  (cond ((pos-visible-in-window-p (point-max))
	 (goto-char (point-max))
	 (recenter -1)
	 (message (substitute-command-keys
		   (if (>= (string-to-int emacs-version) 19)
		       "End.  Type \\[view-exit] to quit viewing."
		     "End.  Type \\[exit-recursive-edit] to quit viewing.")))))
  (move-to-window-line -1)
  (beginning-of-line))

;;
;; Less compatible functions.
;;
(defun View-less-Down (arg)
  (interactive "P")
  "Scrolls up by half window. Compatible to 'd' command of 'Less'."
  (if arg
      (let ((count (prefix-numeric-value arg)))
	(setq view-scroll-size 
	      (if (= count 0) nil count))))
  (View-scroll-lines-forward
   (or view-scroll-size (/ (window-height) 2))))

(defun View-less-Up (arg)
  "Scrolls down by half window. Compatible to 'u' command of 'Less'."
  (interactive "P")
  (if arg
      (let ((count (prefix-numeric-value arg)))
	(setq view-scroll-size 
	      (if (= count 0) nil count))))
  (View-scroll-lines-backward
   (or view-scroll-size (/ (window-height) 2))))

(defun View-goto-max-line ()
  "Goto eob. Compatible to 'G' command of 'Less'."
  (interactive)
  (View-goto-line (point-max)))

(defun scroll-one-line-up (&optional arg)
  "Scroll the selected window up (forward in the text) one line (or N lines)."
  (interactive "p")
  (View-scroll-lines-forward (or arg 1)))

(defun scroll-one-line-down (&optional arg)
  "Scroll the selected window down (backward in the text) one line (or N)."
  (interactive "p")
  (View-scroll-lines-backward (or arg 1)))

;;
;; face-re-search support, Original are func's without 'my-'.
;;
(if (< (string-to-int emacs-version) 19)
    ()
  ;; V19:
  (defun my-View-search-regexp-forward (n regexp)
    "Search forward for Nth occurrence of REGEXP.
Displays line found at center of window.  REGEXP is remembered for
searching with \\[View-search-last-regexp-forward] and \\[View-search-last-regexp-backward].  Sets mark at starting position and pushes mark ring."
    (interactive "p\nsSearch forward (regexp): ")
    (if (> (length regexp) 0)
	(progn
	  ;; Added by s.n. Mon Jul 31 18:53:58 1995
	  (save-excursion
	    ;; View mode has bug (won't repaint face in big buffer).
	    ;; This does not fix, but okay for small buffer.
	    (let ((major-mode view-old-major-mode))
	      (face-re-search-forward regexp)))
	  ;(view-last-command 'View-search-last-regexp-forward n)
	  (view-search n regexp))))
  ;(fset 'my-View-search-regexp-backward 'View-search-regexp-backward)
  (defun my-View-search-regexp-backward (n regexp)
    "Search backward from window start for Nth instance of REGEXP.
Displays line found at center of window.  REGEXP is remembered for
searching with \\[View-search-last-regexp-forward] and \\[View-search-last-regexp-backward].  Sets mark at starting position and pushes mark ring."
    (interactive "p\nsSearch backward (regexp): ")
    (if (> (length regexp) 0)
	(progn
	  ;; Added by s.n. Mon Jul 31 18:53:58 1995
	  (save-excursion
	    ;; View mode has bug (won't repaint face in big buffer).
	    ;; This does not fix, but okay for small buffer.
	    (let ((major-mode view-old-major-mode))
	      (face-re-search-backward regexp)))
	  ;(view-last-command 'View-search-last-regexp-forward n)
	  (view-search (- n) regexp)))
    ;(View-search-regexp-forward (- n) regexp)
    ))

;;
;; 'less' compatible keybindings.
;; This may not be done by hook because this package 'require's view now.
;;
(setq view-hook
      (function
       (lambda ()
	 (define-key view-mode-map "j" 'scroll-one-line-up)
	 (define-key view-mode-map "k" 'scroll-one-line-down)
	 (define-key view-mode-map "J" 'next-line)
	 (define-key view-mode-map "K" 'previous-line)
	 ;; To editor...
	 (define-key view-mode-map "v"
	   (if (>= (string-to-int emacs-version) 19)
	       'view-exit
	     'exit-recursive-edit))

	 ;;  (define-key view-mode-map "j" 'smooth-scroll-up)
	 ;;  (define-key view-mode-map "k" 'smooth-scroll-down)

	 (if (>= (string-to-int emacs-version) 19)
	     (progn 
	       (define-key view-mode-map "l" 'forward-char)

	       ;; Originally this was 'help'.  Rebind to 'H'.
	       (define-key view-mode-map "h" 'backward-char)
	       (define-key view-mode-map "H" 'Helper-describe-bindings))
	   (define-key view-mode-map "l" 'my-forward-char)

	   ;; Originally this was 'help'.  Rebind to 'H'.
	   (define-key view-mode-map "h" 'my-backward-char)
	   (define-key view-mode-map "H" 'Helper-describe-bindings))

	 (define-key view-mode-map "d" 'View-less-Down)
	 (define-key view-mode-map "u" 'View-less-Up)
	 (define-key view-mode-map "G" 'View-goto-max-line)

	 ;; These are rebinding.
	 (if (< (string-to-int emacs-version) 20)
	     (define-key view-mode-map "\C-z" 'View-scroll-lines-backward))
	 (define-key view-mode-map "b" 'View-scroll-lines-backward)
	 (define-key view-mode-map "f" 'View-scroll-lines-forward)

	 (if (>= (string-to-int emacs-version) 19)
	     ;; face-re-search support
	     (progn
	       (define-key view-mode-map "/" 'my-View-search-regexp-forward)
	       (define-key view-mode-map "?" 'my-View-search-regexp-backward)))
)))

(if (>= (string-to-int emacs-version) 20)
    (progn 
      (or (fboundp 'View-scroll-lines-backward)
          (fset 'View-scroll-lines-backward 'View-scroll-line-backward))
      (setq view-mode-hook view-hook)))

(provide 'viewhook)
(provide 'viewhook20)
